// <Copyright>
// ****************************************************************
//                         NOTICE
//
//       CONFIDENTIAL UNPUBLISHED PROPERTY OF UNITED PARCEL
//       SERVICE. USE AND DISTRIBUTION LIMITED SOLELY TO
//       AUTHORIZED PERSONNEL.
//
//
//       The use, disclosure, reproduction, modification,
//       transfer, or transmittal of this work for any purpose
//       in any form or by any means without the written
//       permission of United Parcel Service is strictly
//       prohibited.
//
//       @Copyright 2018 United Parcel Service of America, Inc.
//                        All Rights Reserved
//
//                    DO NOT REPRODUCE OR DUPLICATE
// *****************************************************************
// </Copyright>

const interval = 25;
const radiusInMiles = 3959.0;

const lat_lons=[
    {
        "orig":{"Latitude": 28.541595, "Longitude":-81.368977},
        "dest":{"Latitude": 38.175460, "Longitude":-85.737395}
    },
    {
        "orig":{"Latitude": 35.841241, "Longitude":-78.692780},
        "dest":{"Latitude": 38.175460, "Longitude":-85.737395}
    },
    {
        "orig":{"Latitude": 47.597202, "Longitude":-122.321471},
        "dest":{"Latitude": 38.175460, "Longitude":-85.737395}
    }
];

$(document).ready(function () {

    try {
        createMap();
    } catch (e) {
        alert("Problem creating the map!");
    }

});

function zoomIn(zoomFactor) {
    const map = $("#map");
    const aspectRatio = map.width() / map.height();
    const mapWindow = map.igMap("option", "windowRect");
    const zoomScaleAbs = mapWindow.width * zoomFactor;
    mapWindow.width = mapWindow.width - (2 * zoomScaleAbs);
    mapWindow.height = mapWindow.width / aspectRatio;
    mapWindow.left = Math.min(1.0, mapWindow.left + zoomScaleAbs);
    mapWindow.top = Math.min(1.0, mapWindow.top + zoomScaleAbs);
    map.igMap("option", "windowRect", mapWindow);
}

function panWest(panFactor) {
    const map = $("#map");
    const mapWindow = map.igMap("option", "actualWindowRect");
    const panScale = Math.min(mapWindow.width, mapWindow.height) * panFactor;
    mapWindow.left = Math.max(0.0, mapWindow.left - panScale);
    map.igMap("option", "windowRect", mapWindow);
}


function panNorth(panFactor) {
    const map = $("#map");
    const mapWindow = map.igMap("option", "actualWindowRect");
    const panScale = Math.min(mapWindow.width, mapWindow.height) * panFactor;
    mapWindow.top = Math.max(0.0, mapWindow.top - panScale);
    map.igMap("option", "windowRect", mapWindow);
}

function setDefaultMapView( ) {
    zoomIn(0.4);
    panWest(1.2);
    panNorth(.5);
}

function createMap() {
    $("#map").igMap({
        width: "100%",
        height: "100%",
        windowRect: { left: 0.1, top: 0.1, height: 0.7, width: 0.7 },
        windowResponse: "immediate"
    });
    setDefaultMapView("#map");
    addPaths();
}

function createPaths() {
    const paths = [];
    for (let k = 0; k < lat_lons.length; k++) {
        const pts = calcPath(lat_lons[k].orig, lat_lons[k].dest);
        paths.push({ points: [pts] });
    }
    return paths;
}

function addPaths() {
    $("#map").igMap("option", "series",
        [
            {
                name: "lineSeries",
                type: "geographicPolyline",
                dataSource: createPaths(),
                shapeMemberPath: "points",
                shapeStyle: {
                    stroke: "red",
                    strokeThickness: 2.0
                }
            }
        ]);
}

function calcPath(orig, dest) {
    let path = [];
    let distance = calcDistance(orig, dest);
    if (distance <= interval) {
        path.push({ x: orig.Longitude, y: orig.Latitude,});
        path.push({ x: dest.Longitude, y: dest.Latitude });
    } else {

        let current = orig;
        for (let dist = interval; dist <= distance; dist += interval) {
            path.push({ x: current.Longitude, y: current.Latitude });
            let bearing = calcBearing(current, dest);
            current = calcDestination(current, bearing, interval);
        }
        path.push({ y: dest.Latitude, x: dest.Longitude});
    }
    return path;
}

function calcBearing(orig, dest) {
    orig = getLocation(orig);
    dest = getLocation(dest);
    let dLon = (dest.Longitude - orig.Longitude);
    let y = Math.sin(dLon) * Math.cos(dest.Latitude);
    let x = Math.cos(orig.Latitude) * Math.sin(dest.Latitude) - Math.sin(orig.Latitude) * Math.cos(dest.Latitude) * Math.cos(dLon);
    let angle = Math.atan2(y, x);
    return toDegreesNormalized(angle);
}

function calcDestination(orig, bearing, distance) {

    orig = getLocation(orig);
    bearing = toRadians(bearing);
    distance = distance / radiusInMiles; // angular distance in radians
    let latitude = Math.asin(Math.sin(orig.Latitude) * Math.cos(distance) +
        Math.cos(orig.Latitude) * Math.sin(distance) * Math.cos(bearing));
    let x = Math.sin(bearing) * Math.sin(distance) * Math.cos(orig.Latitude);
    let y = Math.cos(distance) - Math.sin(orig.Latitude) * Math.sin(orig.Latitude);
    let longitude = orig.Longitude + Math.atan2(x, y);
    longitude = (longitude + 3 * Math.PI) % (2 * Math.PI) - Math.PI;  // normalize to -180..+180ยบ
    return { Longitude: toDegrees(longitude), Latitude: toDegrees(latitude) };
}

function calcDistance(orig, dest) {
    orig = getLocation(orig);
    dest = getLocation(dest);
    var sinProd = Math.sin(orig.Latitude) * Math.sin(dest.Latitude);
    var cosProd = Math.cos(orig.Latitude) * Math.cos(dest.Latitude);
    var dLon = (dest.Longitude - orig.Longitude);
    return (Math.acos(sinProd + cosProd * Math.cos(dLon))) * radiusInMiles ;
    //return angle * radiusInMiles; // in miles
}

function getLocation(point) {
    return { Longitude: toRadians(point.Longitude), Latitude: toRadians(point.Latitude) };
}
function toRadians(degrees) {
    return degrees * Math.PI / 180;
}
function toDegrees(radians) {
    return (radians * 180.0 / Math.PI);
}
function toDegreesNormalized(radians) {
    return (toDegrees(radians) + 360) % 360;
}
